//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

module TEST_STACK;

parameter
   WIDTH = 64,
   DEPTH = 4;

reg POP, PUSH;
wire [WIDTH:1] TOS;
reg  [WIDTH:1] TOSD;
assign TOS = TOSD;

//
// Stack
//
STACK PILE(POP, PUSH, TOS);
defparam
   PILE.WIDTH = WIDTH,
   PILE.DEPTH = DEPTH;


//
// Test the stack operation
//
reg [WIDTH:1] DATA [DEPTH:1];
reg [WIDTH:1] TMP;
integer j, k;
initial
begin

   $write("Testing %0d %0d-bit word stack...\n", DEPTH, WIDTH);
   
   // Initialize the test pattern
   for (j = 1; j <= DEPTH; j = j + 1 ) begin
      for (k = 1; k <= WIDTH; k = k + 1 ) begin
         TMP[k] = $random;
      end
      DATA[j] = TMP;
   end

   POP = 1'b0;
   PUSH = 1'b0;
   TOSD = {WIDTH{1'bz}};
   #10;

   // Test with increasing number of data
   for (j = 1; j <= DEPTH; j = j + 1) begin
      for (k = 1; k <= j; k = k + 1) begin
         DO_PUSH(DATA[k]);   
      end
      for (k = j; k >= 1; k = k - 1) begin
         DO_POP(TMP);
         if (TMP !== DATA[k]) begin
            $write("ERROR (%0d/%0d): TOS value is %h instead of %h\n",
                   j, k, TMP, DATA[k]);
         end
      end
   end

   $write("Test completed\n");
   $finish;
end


task DO_POP;
   output [WIDTH:1] PDAT;
begin
   POP = 1'b1;
   #10;
   PDAT = TOS;
   POP  = 1'b0;
   #10;
end
endtask

task DO_PUSH;
   input [WIDTH:1] PDAT;
begin
   TOSD = PDAT;
   PUSH <= #1 1'b1;
   #10;
   PUSH = 1'b0;
   TOSD = {WIDTH{1'bz}};
   #10;
end
endtask

endmodule