//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns/1ns
module DISK_DRIVE(SECT, CYL, DIN, WRITE, GO, DONE, DOUT);

input  [ 3:0] SECT;
input  [ 2:0] CYL;
input  [15:0] DIN;
input         WRITE;
input         GO;
output        DONE;
output [15:0] DOUT;

reg        DONE;
reg [15:0] DOUT;
reg [15:0] DISK [0:{4'b1111, 3'b111}];

//
// Initialize the bad-sector bitmap
//
initial
begin: POWER_UP
   reg [2:0] CYLINDER;

   CYLINDER = 0;
   repeat (8) begin
      // Sector 7 is always good
      DISK[{4'd7, CYLINDER}] = $random | 16'h0080;
      CYLINDER = CYLINDER + 1;
   end

   DONE = 1'b1;
end

//
// Serve requests
//
always @ (posedge GO)
begin: SERVE
   integer d;

   if (WRITE) begin
      // Is this a read-only sector?
      if (SECT != 7) begin
         DISK[{SECT, CYL}] = DIN;
      end
   end
   else begin: READ
      reg [15:0] MASK;

      // Is this a bad sector?
      MASK = DISK[{4'd7, CYL}];
      DOUT =  (MASK[SECT]) ? DISK[{SECT, CYL}] : $random;
   end

   // Terminate the cycle
   d = $random;
   if (d < 0) d = -d;
   #(d % 10);
   DONE = 1'b0;
   @ (negedge GO);
   d = $random;
   if (d < 0) d = -d;
   #(d % 10);
   DONE = 1'b1;
end
endmodule
