//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns/1ns
module TEST_DRIVE;

reg  [ 3:0] SECT;
reg  [ 2:0] CYL;
reg  [15:0] DIN;
reg         WRITE;
reg         GO;
wire        DONE;
wire [15:0] DOUT;

DISK_DRIVE DUT(SECT, CYL, DIN, WRITE, GO, DONE, DOUT);

//
// Test procedure
//
initial
begin: TEST_PROCEDURE
   integer STAMP;
   integer MAX;
   integer AVG;

   $timeformat(-9, 0, " ns", 5);

   GO = 1'b0;
   #10;

   AVG = 0;
   MAX = 0;

   // Perform 25 reads
   WRITE = 1'b0;
   repeat (25) begin
      STAMP = $time;
      GO = 1'b1;
      @ (negedge DONE);
      GO = 1'b0;
      @ (posedge DONE);
      STAMP = $time - STAMP;
      $write("Read seek time  = %t\n", STAMP);
      if (STAMP > MAX) MAX = STAMP;
      AVG = AVG + STAMP;
   end

   // Perform 25 writes
   WRITE = 1'b1;
   repeat (25) begin
      STAMP = $time;
      GO = 1'b1;
      @ (negedge DONE);
      GO = 1'b0;
      @ (posedge DONE);
      STAMP = $time - STAMP;
      $write("Write seek time = %t\n", STAMP);
      if (STAMP > MAX) MAX = STAMP;
      AVG = AVG + STAMP;
   end

   // Report statistics
   AVG = AVG / 50;
   $write("Statistics:\n");
   $write("\tAverage seek time: %t (%0s)\n", AVG,
          (AVG < 10) ? "pass" : "FAIL");
   $write("\tMaximum seek time: %t (%0s)\n", MAX,
          (MAX < 15) ? "pass" : "FAIL");

   $finish;
end



endmodule
