//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns/1ns
module TEST_DFFS;

parameter CK2Q = 2;

reg  D, SI, SM;
reg  CLK, R;
wire Q, Qb;

DFFS DUT(D, SI, SM, CLK, R, Q, Qb);

always
begin
   CLK = 1'b0;
   #50;
   CLK = 1'b1;
   #50;
end

initial
begin
   RESET;
   DATA_IN(1);
   DATA_IN(0);
   DATA_IN(1);
   RESET;
   SCAN_IN(1);
   SCAN_IN(0);
   SCAN_IN(1);
   RESET;
   DATA_IN(1);
   SCAN_IN(0);
   DATA_IN(1);
   DATA_IN(0);
   SCAN_IN(1);
   DATA_IN(0);

   $finish;
end

always
begin
   fork: IS_STABLE
      begin
         @ (posedge CLK);
         disable CATCH;
         #(CK2Q);
      end
      begin: CATCH
         @ (Q or Qb);
         $write("ERROR: Q/Qb not stable\n");
         disable IS_STABLE;
      end
   join
   
end

task RESET;
begin
   R <= 1'b1;
   D <= 1'b1;
   SI <= 1'b1;
   SM <= 1'bx;
   @ (posedge CLK);
   #(CK2Q);
   if (Q !== 1'b0 || Qb !== 1'b1) begin
      $write("ERROR: Invalid reset state: %b%b\n", Q, Qb);
   end
end
endtask

task DATA_IN;
   input DIN;
begin
   R <= 1'b0;
   D <= DIN;
   SI <= ~DIN;
   SM <= 1'b0;
   @ (posedge CLK);
   #(CK2Q);
   if (Q !== DIN || Qb !== ~DIN) begin
      $write("ERROR: Invalid state for DIN=%b: %b%b\n", DIN, Q, Qb);
   end
end
endtask

task SCAN_IN;
   input DIN;
begin
   R <= 1'b0;
   D <= ~DIN;
   SI <= DIN;
   SM <= 1'b1;
   @ (posedge CLK);
   #(CK2Q);
   if (Q !== DIN || Qb !== ~DIN) begin
      $write("ERROR: Invalid state for SI=%b: %b%b\n", DIN, Q, Qb);
   end
end
endtask

endmodule