//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns / 1ns
module TEST_RAM;

reg  [7:0] A;
reg 	   RW;
wire [7:0] D;
reg 	   ADS;
wire       RDY;
wire       INT;
reg 	   CLK;
reg 	   RST;

reg [7:0] DATA;
assign D = DATA;

`ifdef SST
initial $recordvars;
`endif

RAM DUT(A, RW, D, ADS, RDY, CLK);

//
// 16MHz clock
//
always
begin
   CLK = 1'b0;
   #62;
   CLK = 1'b1;
   #62;
end


initial
begin: TEST_PROCEDURE
   reg [7:0] ADDR;
   reg [7:0] TMP;

   $timeformat(-9, 3, " ns", 10);
   ADS  = 1'b1;
   DATA = 8'hZZ;

   ADDR = 0;
   WRITE(ADDR, ADDR);
   repeat (256) begin
      ADDR = ADDR + 1;
      WRITE(ADDR, ADDR);
      READ(ADDR-1, TMP);
      if (TMP !== ADDR - 8'd1) begin
         $write("ERROR: RAM[%0d] = %h instead of %h\n",
                ADDR-8'd1, TMP, ADDR-8'd1);
      end
   end

   $finish;   
end

time LAST_RDY;
always @ (RDY) LAST_RDY = $time;
time LAST_D;
always @ (D) LAST_D = $time;


//
// Read Cycle
//
task READ;
   input  [23:1] ADDR;
   output [15:0] RDAT;
   
begin
   @ (posedge CLK);
   A   <= #4 ADDR;
   RW  <= #4 1'b0;
   ADS <= #4 1'b0;

   @ (posedge CLK);
   
   @ (posedge CLK);
   ADS <= #4 1'b1;

   repeat (2) @ (posedge CLK);
   while (RDY !== 1'b0) begin
      repeat (2) @ (posedge CLK);
   end
   if ($time - LAST_RDY < 19) begin
      $write("WARNING: Setup violation on RDY at %t\n", $realtime);
   end

   if ($time - LAST_D < 9) begin
      $write("WARNING: Setup violation on D at %t\n", $realtime);
   end
   RDAT = D;

   fork
      begin
	 #4;
	 if ($time - LAST_RDY < 4+1) begin
	    $write("WARNING: Hold violation on RDY at %t\n", $realtime);
	 end
      end
      begin
	 #6;
	 if ($time - LAST_D < 6+1) begin
	    $write("WARNING: Hold violation on D at %t\n", $realtime);
	 end
      end
   join
end
endtask

//
// Write Cycle
//
assign #(4) D = DATA;
task WRITE;
   input [23:1] ADDR;
   input [15:0] WDAT;
   
begin
   @ (posedge CLK);   
   A    <= #4 ADDR;
   RW   <= #4 1'b1;
   ADS  <= #4 1'b0;

   @ (posedge CLK);
   DATA <= #4 WDAT;

   @ (posedge CLK);
   ADS <= #4 1'b1;

   repeat (2) @ (posedge CLK);
   while (RDY !== 1'b0) begin
      repeat (2) @ (negedge CLK);
   end
   if ($time - LAST_RDY < 19) begin
      $write("WARNING: Setup violation on RDY at %t\n", $realtime);
   end
   #4;
   if ($time - LAST_RDY < 4+1) begin
      $write("WARNING: Hold violation on RDY at %t\n", $realtime);
   end

   @ (posedge CLK);
   DATA <= #4 8'hZZ;
end
endtask


endmodule
