//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns/1ns
module THING(A, WR, D, ADS, READY, INT, CLK, RST);

//
// i386SX CPU Interface
//
input  [7:0] A;
input        WR;
inout  [7:0] D;
input 	     ADS;
output 	     READY;

//
// Control Interface
//
output 	INT;
input 	CLK;
input 	RST;

//
// i386SX CPU interface
//
reg 	   READY;
reg [7:0]  STATUS, MASK, DATA;
assign D = DATA;

always
begin: i386
   DATA  <= #7 8'hZZ;
   READY <= #5 1'b1;

   
   // Phase 1
   @ (posedge CLK)
   if (ADS !== 1'b0) disable i386;
   // Phase 2
   @ (posedge CLK)
   if (ADS !== 1'b0) begin
      $write("ERROR: ADS not asserted during phase 2 at %t\n", $realtime);
   end
   // Phase 1
   @ (posedge CLK)
   if (ADS !== 1'b1) begin
      $write("ERROR: ADS not deasserted during phase 1 at %t\n", $realtime);
   end
   
   READY <= #5 1'b0;

   if (WR == 1'b1) begin
      // Write cycle
      @ (posedge CLK);
      if (A === 8'h01) begin
	 MASK <= D;
      end
   end
   else begin
      // Read cycle
      case (A)
       8'h00: DATA <= #5 STATUS;
       8'h01: DATA <= #5 MASK;
      endcase
      @ (posedge CLK);
   end
end

assign INT = |(STATUS & ~MASK);

always
begin: RUN_ONCE
   #2000 STATUS = 8'h04;
   #4000 STATUS = 8'h05;
   #2000 STATUS = 8'h07;
   wait (0);
end

always
begin
   wait (RST === 1'b1);
   disable RUN_ONCE;
   disable i386;
   MASK   = 8'h00;
   STATUS = 8'h00;
   wait (RST !== 1'b1);
   disable RUN_ONCE;
   disable i386;
   MASK   = 8'h00;
   STATUS = 8'h00;
end

endmodule
