//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns / 1ns
module TEST_THING;

reg  [7:0] A;
reg 	   RW;
wire [7:0] D;
reg 	   ADS;
wire       RDY;
wire       INT;
reg 	   CLK;
reg 	   RST;

reg [7:0] DATA;
assign D = DATA;

`ifdef SST
initial $recordvars;
`endif

`ifdef SDF
initial $sdf_annotate("thing.sdf", DUT);
`endif

THING DUT(.A(A),
	  .WR(RW),
	  .D(D),
	  .ADS(ADS),
	  .READY(RDY),
	  .INT(INT),
	  .CLK(CLK),
	  .RST(RST));

//
// 16MHz clock
//
always
begin
   CLK = 1'b0;
   #62;
   CLK = 1'b1;
   #62;
end


// Monitor the interrupt output
always
begin
   wait (INT === 1'b1);
   $write("INT asserted at %0t\n", $realtime);
   wait (INT !== 1'b1);
end
 
initial
begin: TEST_PROCEDURE
   reg [2:0] SEEN;
   reg [7:0] TMP;

   $timeformat(-9, 3, " ns", 10);
   ADS  = 1'b1;
   DATA = 8'hZZ;
   SEEN = 3'b000;

   RST = 1'b1;
   repeat (4) @ (posedge CLK);
   RST = 1'b0;

   while (SEEN != 3'b111) begin
      wait (INT != 1'b0);

      READ(0, TMP);
      TMP = TMP & ~SEEN;

      if (TMP[2] == 1'b1) $write("A has been set\n");
      if (TMP[1] == 1'b1) $write("B has been set\n");
      if (TMP[0] == 1'b1) $write("C has been set\n");

      SEEN = SEEN | TMP;
      WRITE(1, SEEN);
   end

   $finish;   
end

parameter
   t6    = 4,
   t10   = 4,
   t12   = 4,
   t13   = 4,
   t19   = 19,
   t20   = 4,
   t21   = 9,
   t22   = 6;

time LAST_RDY;
always @ (RDY) LAST_RDY = $time;
time LAST_D;
always @ (D) LAST_D = $time;


//
// Read Cycle
//
task READ;
   input  [23:1] ADDR;
   output [15:0] RDAT;
   
begin
   @ (posedge CLK);
   A   <= #4 ADDR;
   RW  <= #4 1'b0;
   ADS <= #4 1'b0;

   @ (posedge CLK);
   
   @ (posedge CLK);
   ADS <= #4 1'b1;

   repeat (2) @ (posedge CLK);
   while (RDY !== 1'b0) begin
      repeat (2) @ (posedge CLK);
   end
   if ($time - LAST_RDY < 19) begin
      $write("WARNING: Setup violation on RDY at %t\n", $realtime);
   end

   if ($time - LAST_D < 9) begin
      $write("WARNING: Setup violation on D at %t\n", $realtime);
   end
   RDAT = D;

   fork
      begin
	 #4;
	 if ($time - LAST_RDY < 4+1) begin
	    $write("WARNING: Hold violation on RDY at %t\n", $realtime);
	 end
      end
      begin
	 #6;
	 if ($time - LAST_D < 6+1) begin
	    $write("WARNING: Hold violation on D at %t\n", $realtime);
	 end
      end
   join
end
endtask

//
// Write Cycle
//
assign #(t12, t12, t13) D = DATA;
task WRITE;
   input [23:1] ADDR;
   input [15:0] WDAT;
   
begin
   @ (posedge CLK);   
   A    <= #4 ADDR;
   RW   <= #4 1'b1;
   ADS  <= #4 1'b0;

   @ (posedge CLK);
   DATA <= #4 WDAT;

   @ (posedge CLK);
   ADS <= #4 1'b1;

   repeat (2) @ (posedge CLK);
   while (RDY !== 1'b0) begin
      repeat (2) @ (negedge CLK);
   end
   if ($time - LAST_RDY < 19) begin
      $write("WARNING: Setup violation on RDY at %t\n", $realtime);
   end
   #4;
   if ($time - LAST_RDY < 4+1) begin
      $write("WARNING: Hold violation on RDY at %t\n", $realtime);
   end

   @ (posedge CLK);
   DATA <= #4 8'hZZ;
end
endtask


endmodule
