//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns/1ns
module THING(A, WR, D, ADS, READY, INT, CLK, RST);

//
// i386SX CPU Interface
//
input  [7:0] A;
input        WR;
inout  [7:0] D;
input 	     ADS;
output 	     READY;

//
// Control Interface
//
output 	INT;
input 	CLK;
input 	RST;

//
// i386SX CPU interface
//
reg [7:0]  STATUS, MASK, DATA;

reg [1:0]  STATE;
parameter IDLE = 2'b00,
	  STRT = 2'b01,
	  PHI1 = 2'b11,
	  RDY  = 2'b10;
	  
always @ (posedge CLK)
begin: i386
   if (RST == 1'b1) begin
      STATE <= IDLE;
      MASK  <= 8'h00;
   end
   else begin
      case (STATE)
       IDLE: begin
	  if (ADS == 1'b0) begin
	     STATE <= STRT;
	  end
       end
       STRT: begin
	  STATE <= PHI1;
       end
       PHI1: begin
	  STATE <= RDY;
	  if (WR == 1'b0) begin
	     case (A) // synopsys full_case
	      8'h00: DATA <= STATUS;
	      8'h01: DATA <= MASK;
	     endcase
	  end
       end
       RDY: begin
	  STATE <= IDLE;
	  if (WR == 1'b1) begin
	     case (A) // synopsys full_case
	      8'h00: ;
	      8'h01: MASK <= D;
	     endcase
	  end
       end
      endcase
   end
end
assign #5 D     = (STATE == RDY) ? DATA : 8'hZZ;
assign #7 READY = (STATE == RDY) ? 1'b0 : 1'b1;

assign INT = |(STATUS & ~MASK);


//
// Generate spurious interrupts
//
reg [7:0] COUNT;
always @ (posedge CLK)
begin
   if (RST == 1'b1) begin
      STATUS <= 8'h00;
      COUNT  <= 0;
   end
   else begin
      if (COUNT == 8'hFF) begin
	 // A, C, then B
	 casez (STATUS[2:0])
	  3'b??0: STATUS[0] <= 1'b1;
	  3'b0?1: STATUS[2] <= 1'b1;
	  3'b101: STATUS[1] <= 1'b1;
	 endcase
      end
      COUNT <= COUNT + 1;
   end
end

endmodule
