//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

module TB;

reg	[3:0]	OP;
reg	[2:0]	A;
reg	[2:0]	B;
wire	[2:0]	C;
wire		T;

LOGIC_UNIT UUT(OP, A, B, C, T);

initial
begin: TEST_PROCEDURE
    integer ERRORS;
    
    ERRORS = 0;

    $write("Testing the 'AND' opcode with 16 pairs of random input values\n");
    OP = 4'b0000;
    repeat (16) begin
        A = $random;
        B = $random;
        #100;
        if (C !== (A & B)) begin
            $write("* ERROR * 'b%b & 'b%b returns 'b%b instead of 'b%b\n",
                   A, B, C, A & B);
            ERRORS = ERRORS + 1;
        end
    end
    
    $write("Testing the 'OR' opcode with 16 pairs of random input values\n");
    OP = 4'b0001;
    repeat (16) begin
        A = $random;
        B = $random;
        #100;
        if (C !== (A | B)) begin
            $write("* ERROR * 'b%b | 'b%b returns 'b%b instead of 'b%b\n",
                   A, B, C, A | B);
            ERRORS = ERRORS + 1;
        end
    end
    
    $write("Testing the 'A<0' opcode with 16 pairs of random input values\n");
    OP = 4'b1000;
    repeat (16) begin
        A = $random;
        B = $random;
        #100;
        if (T !== A[2]) begin
            $write("* ERROR * 'b%b returns 'b%b instead of 'b%b\n",
                   A, T, A[2]);
            ERRORS = ERRORS + 1;
        end
    end
    
    $write("Testing the 'B<0' opcode with 16 pairs of random input values\n");
    OP = 4'b1001;
    repeat (16) begin
        A = $random;
        B = $random;
        #100;
        if (T !== B[2]) begin
            $write("* ERROR * 'b%b returns 'b%b instead of 'b%b\n",
                   B, T, B[2]);
            ERRORS = ERRORS + 1;
        end
    end
    
    if (ERRORS > 0) $write("Your model has failed with %0d errors\n", ERRORS);
    else $write("Your model has passed!\n");
end
endmodule
