//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

module XBAR(I, O, R1, R2, R3);

input	[1:3]	I;
output	[1:3]	O;
input	[1:0]   R1;
input	[1:0]   R2;
input	[1:0]   R3;

wire	[1:3]	O;

//
// Please specify a synthesizable description of a non-blocking
// fully connected 3x3 crossbar using tri-state buffers.
// Contention will be possible.
// 
// The connection maps for inputs I[1], I[2], and I[3] are given by
// the inputs R1, R2, and R3, respectively.
// 
// The connection to output buses is as specified:
//
//      Rn      I[n]
//----------------------------
// 	00	Unconnected
// 	01	output 1
// 	10	output 2
// 	11	output 3
// 

  assign O[1] = ((R1 == 2'b01) ? I[1] : 1'bz);
  assign O[2] = ((R1 == 2'b10) ? I[1] : 1'bz);
  assign O[3] = ((R1 == 2'b11) ? I[1] : 1'bz);

  assign O[1] = ((R2 == 2'b01) ? I[2] : 1'bz);
  assign O[2] = ((R2 == 2'b10) ? I[2] : 1'bz);
  assign O[3] = ((R2 == 2'b11) ? I[2] : 1'bz);

  assign O[1] = ((R3 == 2'b01) ? I[3] : 1'bz);
  assign O[2] = ((R3 == 2'b10) ? I[3] : 1'bz);
  assign O[3] = ((R3 == 2'b11) ? I[3] : 1'bz);

endmodule
