//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

module TB;

reg	[1:3]	I;
wire	[1:3]	O;
reg	[0:5]	C;

XBAR UUT(I, O, C);


function [0:5] CONNECT;
    input	[0:1]	A;
    input	[0:1]	B;
    input	[0:1]	C;
begin
    CONNECT = {A, B, C};
end
endfunction

function RESULT;
    input	EXPECT;
    input	I;
begin
    if (EXPECT === 1'bz || EXPECT === I) RESULT = I;
    else RESULT = 1'bx;
end
endfunction


initial
begin: TEST_PROCEDURE
    integer ERRORS;
    
    reg	[1:3]	EXPECT;
    integer i, j, k;
    
    $timeformat(-9, 0, " ns", 0);
    
    ERRORS = 0;
    
    I = 3'b01x;
    
    $write("Testing all possible connections\n");
    for (i = 0; i < 4; i = i + 1) begin
        for (j = 0; j < 4; j = j + 1) begin
            for (k = 0; k < 4; k = k + 1) begin
                C = CONNECT(i, j, k);
                EXPECT = 3'bzzz;
                if (i > 0) EXPECT[i] = RESULT(EXPECT[i], I[1]);
                if (j > 0) EXPECT[j] = RESULT(EXPECT[j], I[2]);
                if (k > 0) EXPECT[k] = RESULT(EXPECT[k], I[3]);
                #100;
                if (O !== EXPECT) begin
                    $write("* ERROR * Connect map 'b%b with input 'b%b gave 'b%b instead of 'b%b at %t\n",
                           C, I, O, EXPECT, $time);
                    ERRORS = ERRORS + 1;
                end
            end
        end
    end
    
    if (ERRORS > 0) $write("Your model has failed with %0d errors\n", ERRORS);
    else $write("Your model has passed!\n");
end

endmodule
