//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

module TB;

reg	DI;
wire	DO;
reg	CLK;

SHIFTER UUT(DI, DO, CLK);

parameter
    LENGTH = 58,
    DEPTH  = 4;

reg	[1:LENGTH]	PATTERN;
initial	PATTERN = 58'b0000100001100001110000111100001111011110011110001111010101;

initial
begin: APPLY
    integer	i;
    
    $write("Testing shifter for %0d cycles\n", LENGTH);
    $write("%b\n", PATTERN);
    for (i = 1; i <= LENGTH; i = i + 1) begin
        @ (negedge CLK);
        DI = PATTERN[i];
    end
end

initial
begin: VERIFY
    integer	i;
    integer	ERRORS;
    
    ERRORS = 0;
    
    // Synchronize with the data input
    @ (negedge CLK);
    
    // Wait for the initial content to be flushed
    // and the first data bit to appear
    repeat (DEPTH - 1) @ (negedge CLK);
    
    // The first bit should be at the output
    // after the next clock edge
    for (i = 1; i <= LENGTH; i = i + 1) begin
        @ (negedge CLK);
        
        if (PATTERN[i] !== DO) begin
            $write("* ERROR * Output bit #%0d was %0b instead of %0b\n",
                   i, DO, PATTERN[i]);
            ERRORS = ERRORS + 1;
        end
    end
    
    if (ERRORS > 0) $write("Your model has failed with %0d errors\n", ERRORS);
    else $write("Your model has passed!\n");
    
    $finish;
end

always
begin
    CLK = 1'b0;
    #50;
    CLK = 1'b1;
    #50;
end

endmodule
