//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

module TB;

reg	[3:0]	I;
reg		LD;
reg		RT;
reg		LF;
reg		CK;
reg		RST;
wire	[3:0]	O;

RT_LF_SHIFT UUT(I, LD, RT, LF, CK, RST, O);

initial
begin: TEST_PROCEDURE
    reg	[3:0]	EXPECT;
    integer	ERRORS;
    
    ERRORS = 0;
    
    LD = 1'b0;
    RT = 1'b0;
    LF = 1'b0;
    RST = 1'b0;

    @ (negedge CK);

    $write("Testing parallel load...\n");
    LD = 1'b1;

    I = 4'b1111;
    @ (negedge CK);
    if (O !== I) begin
        $write("* ERROR * Loaded value is %b instead of %b at %t\n",
               O, I, $time);
        ERRORS = ERRORS + 1;
    end
    I = 4'b1010;
    @ (negedge CK);
    if (O !== I) begin
        $write("* ERROR * Loaded value is %b instead of %b at %t\n",
               O, I, $time);
        ERRORS = ERRORS + 1;
    end
    I = 4'b0101;
    @ (negedge CK);
    if (O !== I) begin
        $write("* ERROR * Loaded value is %b instead of %b at %t\n",
               O, I, $time);
        ERRORS = ERRORS + 1;
    end
    I = 4'b0000;
    @ (negedge CK);
    if (O !== I) begin
        $write("* ERROR * Loaded value is %b instead of %b at %t\n",
               O, I, $time);
        ERRORS = ERRORS + 1;
    end

    $write("Testing sync reset...\n");
    I = 4'b1111;
    @ (negedge CK);
    LD = 1'b0;
    RST = 1'b1;
    #10 RST = 1'b0;
    if (O !== I) begin
        $write("* ERROR * Reset seems to be asynchronous at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    RST = 1'b1;
    @ (negedge CK);
    if (O !== 4'b0000) begin
        $write("* ERROR * Reset did not work. Output is %b at %t\n",
               O, $time);
        ERRORS = ERRORS + 1;
    end
    RST = 1'b0;
    
    $write("Testing shift operations...\n");
    I = 4'b0001;
    LD = 1'b1;
    repeat (3) begin
        @ (negedge CK);
        LD = 1'b0;
        if (O !== I) begin
            $write("* ERROR * Output is not stable: %b instead of %b at %t\n",
                   O, I, $time);
            ERRORS = ERRORS + 1;
        end
    end
    LF = 1'b1;
    EXPECT = I;
    repeat (3) begin
        @ (negedge CK);
        EXPECT = EXPECT << 1;
        if (O !== EXPECT) begin
            $write("* ERROR * Output is not as expected: %b instead of %b at %t\n",
                   O, EXPECT, $time);
            ERRORS = ERRORS + 1;
        end
    end
    LF = 1'b0;
    RT = 1'b1;
    repeat (4) begin
        @ (negedge CK);
        EXPECT = EXPECT >> 1;
        if (O !== EXPECT) begin
            $write("* ERROR * Output is not as expected: %b instead of %b at %t\n",
                   O, EXPECT, $time);
            ERRORS = ERRORS + 1;
        end
    end
    RT = 1'b0;
    I = 4'b1000;
    LD = 1'b1;
    repeat (3) begin
        @ (negedge CK);
        LD = 1'b0;
        if (O !== I) begin
            $write("* ERROR * Output is not stable: %b instead of %b at %t\n",
                   O, I, $time);
            ERRORS = ERRORS + 1;
        end
    end
    LF = 1'b1;
    EXPECT = I;
    @ (negedge CK);
    EXPECT = EXPECT << 1;
    if (O !== EXPECT) begin
        $write("* ERROR * Output is not as expected: %b instead of %b at %t\n",
               O, EXPECT, $time);
        ERRORS = ERRORS + 1;
    end
    
    if (ERRORS > 0) $write("Your model has failed with %0d errors\n", ERRORS);
    else $write("Your model has passed!\n");

    $finish;
end

always
begin
    CK = 1'b0;
    #50;
    CK = 1'b1;
    #50;
end

endmodule
