//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

module TB;

reg	I;
wire	R;
wire	F;
reg	CK;

EDGE_DETECTOR UUT (I, R, F, CK);

integer	R_SEEN;
integer	F_SEEN;
initial
begin
    R_SEEN = 0;
    F_SEEN = 0;
end
always @ (posedge CK) if (R == 1'b1) R_SEEN = R_SEEN + 1;
always @ (posedge CK) if (F == 1'b1) F_SEEN = F_SEEN + 1;

initial
begin: TEST_PROCEDURE
    integer	LAST_R;
    integer	LAST_F;
    integer	ERRORS;
    
    ERRORS = 0;
    
    // Let the detector flush out x's
    I = 0;
    repeat (8) @ (negedge CK);
    
    $write("Testing with a stable '0'\n");
    I = 0;
    LAST_R = R_SEEN;
    LAST_F = F_SEEN;
    repeat (8) @ (negedge CK);
    if (LAST_R !== R_SEEN) begin
        $write("* ERROR * Rising edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F !== F_SEEN) begin
        $write("* ERROR * Falling edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    
    $write("Testing with a rising edge\n");
    I = 1;
    LAST_R = R_SEEN;
    LAST_F = F_SEEN;
    repeat (4) @ (negedge CK);
    if (LAST_R == R_SEEN) begin
        $write("* ERROR * No rising edge was detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_R > R_SEEN + 1) begin
        $write("* ERROR * More than one rising edge was detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F !== F_SEEN) begin
        $write("* ERROR * Falling edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    
    $write("Testing with a stable '1'\n");
    I = 1;
    LAST_R = R_SEEN;
    LAST_F = F_SEEN;
    repeat (8) @ (negedge CK);
    if (LAST_R !== R_SEEN) begin
        $write("* ERROR * Rising edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F !== F_SEEN) begin
        $write("* ERROR * Falling edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    
    $write("Testing with a falling edge\n");
    I = 0;
    LAST_R = R_SEEN;
    LAST_F = F_SEEN;
    repeat (4) @ (negedge CK);
    if (LAST_R !== R_SEEN) begin
        $write("* ERROR * Rising edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F == F_SEEN) begin
        $write("* ERROR * No Falling edge was detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F > F_SEEN + 1) begin
        $write("* ERROR * More than one falling edge was detected\n");
        ERRORS = ERRORS + 1;
    end
    
    $write("Testing with a stable '0'\n");
    I = 0;
    LAST_R = R_SEEN;
    LAST_F = F_SEEN;
    repeat (8) @ (negedge CK);
    if (LAST_R !== R_SEEN) begin
        $write("* ERROR * Rising edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F !== F_SEEN) begin
        $write("* ERROR * Falling edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    
    $write("Testing with a '1' glitch\n");
    I = 0;
    I <= #10 1;
    I <= #20 0;
    LAST_R = R_SEEN;
    LAST_F = F_SEEN;
    repeat (8) @ (negedge CK);
    if (LAST_R !== R_SEEN) begin
        $write("* ERROR * Rising edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F !== F_SEEN) begin
        $write("* ERROR * Falling edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    
    $write("Testing with a '0' glitch\n");
    I = 1;
    repeat (8) @ (negedge CK);
    I <= #10 0;
    I <= #20 1;
    LAST_R = R_SEEN;
    LAST_F = F_SEEN;
    repeat (8) @ (negedge CK);
    if (LAST_R !== R_SEEN) begin
        $write("* ERROR * Rising edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end
    if (LAST_F !== F_SEEN) begin
        $write("* ERROR * Falling edge(s) was(were) detected\n");
        ERRORS = ERRORS + 1;
    end

    if (ERRORS > 0) $write("Your model has failed with %0d errors\n", ERRORS);
    else $write("Your model has passed!\n");

    $finish;
end

always
begin
    CK = 1'b0;
    #50;
    CK = 1'b1;
    #50;
end
endmodule
