//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

`define cycle 100
module TB;

reg		CK;
wire	[2:0]	CNT;
reg		RST;

GRAY_COUNT UUT(CK, CNT, RST);

initial begin
    CK = 1'b0;
    forever #(`cycle/2) CK = ~CK;
end

initial $timeformat(-9,0,"ns", 14);

initial begin: TEST_PROCEDURE
    reg	[2:0]	CODE	[0:7];
    reg	[2:0]	TMP;
    integer	i, j, k;
    integer	ERRORS;
    
    ERRORS = 0;
    
    $write("Reseting the counter\n");
    RST = 1'b1;
    repeat (2) @ (negedge CK);
    
    $write("Checking gray code sequence\n");
    RST = 1'b0;
    CODE[0] = CNT;
    for (i = 1; i < 8; i = i + 1) begin
        @ (negedge CK);
        CODE[i] = CNT;
    end

    // Verify that all codes are unique
    for (i = 0; i < 7; i = i + 1) begin
        for (j = i + 1; j < 8; j = j + 1) begin
            if (CODE[i] === CODE[j]) begin
                $write("* ERROR * Codes #%0d & #%0d are identical: %b\n", i, j, CODE[i]);
                ERRORS = ERRORS + 1;
            end
        end
    end

    // Verify that it is indeed a gray code
    for (i = 1; i < 8; i = i + 1) begin
        TMP = CODE[i-1] ^ CODE[i];
        // There should be only 1 bit set
        k = 0;
        for (j = 0; j < 3; j = j + 1) k = k + TMP[j];
        if (k !== 1) begin
            $write("* ERROR * There are %0d different bits between codes #%0d (%b) and #%0d (%b)\n",
                   k, i-1, CODE[i-1], i, CODE[i]);
            ERRORS = ERRORS + 1;
        end
    end

    // Make sure code wraps back
    @ (negedge CK);
    if (CNT !== CODE[0]) begin
        $write("* ERROR * Code did not wrap back to reset value: %b instead of %b at %0t\n",
               CNT, CODE[0], $time);
        ERRORS = ERRORS + 1;
    end

    // Verify that sequence repeats
    for (i = 1; i < 8; i = i + 1) begin
        @ (negedge CK);
        if (CNT !== CODE[i]) begin
            $write("* ERROR * Code did not repeat: %b instead of %b at %0t\n",
                   CNT, CODE[i], $time);
            ERRORS = ERRORS + 1;
        end
    end
    
    if (ERRORS > 0) $write("Your model has failed with %0d errors\n", ERRORS);
    else $write("Your model has passed!\n");

    $finish;
end

endmodule
