//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

module FIFO(DI, CKI, DO, CKO, FULL, MT, RST);

input	[7:0]	DI;
input		CKI;
output	[7:0]	DO;
input		CKO;
output		FULL;
output		MT;
input		RST;

reg		[7:0] FIFOREG [0:3];
reg		[1:0] WRITE_PTR, READ_PTR;

//
// Please specify a synthesizable description of an asynchrosnously
// resettable FIFO.
// Words are written at the rising edge of CKI and are consumed
// at the rising edge of CKO.
// The outputs FULL and MT are asserted high when the FIFO is full
// and empty respectively.
// 
// The testbench is set-up for a 3-deep FIFO. To test a FIFO with
// a different number of words, simply change the value of 'DEPTH'
// and provide additional CONTENT values to test all the locations.
// 

  always @(posedge CKI or posedge RST) begin
	if (RST == 1'b1)
	  WRITE_PTR <= 2'b0;
	else begin
	  if (FULL == 1'b1);
      else begin
		FIFOREG[WRITE_PTR] <= DI;
        WRITE_PTR <= WRITE_PTR + 1;
      end
	end
  end

  always @(posedge CKO or posedge RST) begin
	if (RST == 1'b1)
	  READ_PTR <= 2'b0;
	else begin
	  if (MT == 1'b1);
	  else READ_PTR <= READ_PTR + 1;
	end
  end

  assign DO   = FIFOREG[READ_PTR];

  assign MT   = (READ_PTR == WRITE_PTR);

  assign FULL = ((WRITE_PTR + 1) == READ_PTR) ? 1'b1 : 1'b0;

endmodule
