//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1 ns / 1 ns

module TB;

reg	[7:0]	DI;
reg		CKI;
wire	[7:0]	DO;
reg		CKO;
wire		FULL;
wire		MT;
reg		RST;

FIFO UUT(DI, CKI, DO, CKO, FULL, MT, RST);


task WRITE;
begin
    #2  CKI = 1'b1;
    #10 CKI = 1'b0;
end
endtask

task READ;
begin
    #2  CKO = 1'b1;
    #10 CKO = 1'b0;
end
endtask

parameter
    DEPTH = 3;

initial
begin: TEST_PROCEDURE
    integer ERRORS;
    reg	[7:0]	CONTENT	[0:DEPTH];
    integer	i;
    
    $timeformat(-9, 0, " ns", 0);
    
    ERRORS = 0;
    
    CONTENT[0] = 8'hAA;
    CONTENT[1] = 8'h55;
    CONTENT[2] = 8'h00;
    CONTENT[3] = 8'hFF;
    
    CKI = 0;
    CKO = 0;
    RST = 1'b0;
    
    $write("Resetting the FIFO\n");
    RST = 1'b1;
    #50;
    RST = 1'b0;
    if (MT !== 1'b1) begin
        $write("* ERROR * FIFO is not empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    
    i = 0;
    $write("Writing 1 value in FIFO\n");
    DI = CONTENT[i];
    WRITE;
    if (MT !== 1'b0) begin
        $write("* ERROR * FIFO is empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end

    $write("Reading 1 value from FIFO\n");
    if (DO !== CONTENT[i]) begin
        $write("* ERROR * Value #%0d in FIFO is %h instead of %h at %t\n",
               i, DO, CONTENT[i], $time);
        ERRORS = ERRORS + 1;
    end
    READ;
    if (MT !== 1'b1) begin
        $write("* ERROR * FIFO is not empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end

    $write("Writing 1 value in FIFO\n");
    DI = CONTENT[i];
    WRITE;
    if (MT !== 1'b0) begin
        $write("* ERROR * FIFO is empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end

    $write("Writing %0d value in FIFO\n", DEPTH - 2);
    repeat (DEPTH - 2) begin
        i = i + 1;
        DI = CONTENT[i];
        WRITE;
        if (MT !== 1'b0) begin
            $write("* ERROR * FIFO is empty at %t\n", $time);
            ERRORS = ERRORS + 1;
        end
        if (FULL !== 1'b0) begin
            $write("* ERROR * FIFO is full at %t\n", $time);
            ERRORS = ERRORS + 1;
        end
    end

    $write("Writing 1 value in FIFO\n");
    i = i + 1;
    DI = CONTENT[i];
    WRITE;
    if (MT !== 1'b0) begin
        $write("* ERROR * FIFO is empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b1) begin
        $write("* ERROR * FIFO is not full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    
    $write("\nFlushing the FIFO\n\n");
    
    i = 0;
    $write("Reading 1 value from FIFO\n");
    if (DO !== CONTENT[i]) begin
        $write("* ERROR * Value #%0d in FIFO is %h instead of %h at %t\n",
               i, DO, CONTENT[i], $time);
        ERRORS = ERRORS + 1;
    end
    READ;
    if (MT !== 1'b0) begin
        $write("* ERROR * FIFO is empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end

    $write("Reading %0d value from FIFO\n", DEPTH - 2);
    repeat (DEPTH - 2) begin
        i = i + 1;
        if (DO !== CONTENT[i]) begin
            $write("* ERROR * Value #%0d in FIFO is %h instead of %h at %t\n",
                   i, DO, CONTENT[i], $time);
            ERRORS = ERRORS + 1;
        end
        READ;
        if (MT !== 1'b0) begin
            $write("* ERROR * FIFO is empty at %t\n", $time);
            ERRORS = ERRORS + 1;
        end
        if (FULL !== 1'b0) begin
            $write("* ERROR * FIFO is full at %t\n", $time);
            ERRORS = ERRORS + 1;
        end
    end

    $write("Reading 1 value from FIFO\n");
    i = i + 1;
    if (DO !== CONTENT[i]) begin
        $write("* ERROR * Value #%0d in FIFO is %h instead of %h at %t\n",
               i, DO, CONTENT[i], $time);
        ERRORS = ERRORS + 1;
    end
    READ;
    if (MT !== 1'b1) begin
        $write("* ERROR * FIFO is not empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end

    $write("\nTesting reset again\n\n");
    
    $write("Writing 1 value in FIFO\n");
    WRITE;
    if (MT !== 1'b0) begin
        $write("* ERROR * FIFO is empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end

    $write("Resetting the FIFO\n");
    RST = 1'b1;
    #50;
    RST = 1'b0;
    if (MT !== 1'b1) begin
        $write("* ERROR * FIFO is not empty at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    if (FULL !== 1'b0) begin
        $write("* ERROR * FIFO is full at %t\n", $time);
        ERRORS = ERRORS + 1;
    end
    
    if (ERRORS > 0) $write("Your model has failed with %0d errors\n", ERRORS);
    else $write("Your model has passed!\n");

    $finish;
end

endmodule
